/*
  SFBWatchdog.cpp - Host side stubs for watchdog reset management routines
  Copyright (C) 2009 The Regents of the University of New Mexico.  All rights reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  USA

  $Id$
*/

#include "SFBWatchdog.h"
#include <stdlib.h>             // For exit()

bool watchdogActive() {
  return false;
}

void watchdogSetInterval(u32 feedIntervalMs) { /* ignore */ }

u32 watchdogGetInterval() { return 0; }

bool watchdogStart(u32 feedIntervalMs) {
  return false;                 // We don't do that heah
}

bool watchdogFeed() {
  return false;                 // ..nothing to feed
}

bool watchdogGetTimeoutFlag() {
  return false;
}

void watchdogClearTimeoutFlag() {
  /* nothing */
}

void watchdogSuicide() {        
  exit(23);                     // watchdogSuicide promises not to return..
}

